// src/index.ts
import SyntaxJSX from "@babel/plugin-syntax-jsx";
import { parse as parse2 } from "@babel/parser";

// src/transform.ts
import { compile } from "@vue-jsx-vapor/compiler";
import { parse } from "@babel/parser";
import { SourceMapConsumer } from "source-map-js";
import traverse from "@babel/traverse";
var transformJSX = (path, state) => {
  const { parent, node } = path;
  if (!(parent?.type !== "JSXExpressionContainer" && !isJSXElement(parent))) {
    return;
  }
  let { code, vaporHelpers, preamble, map } = compile(
    state.rootCodes.shift(),
    {
      mode: "module",
      inline: true,
      isTS: state.filename?.endsWith("tsx"),
      filename: state.filename,
      sourceMap: true,
      ...state?.compile
    }
  );
  vaporHelpers.forEach((helper) => state.importSet.add(helper));
  preamble = preamble.replaceAll(
    /(?<=const )t(?=(\d))/g,
    `_t${state.preambleIndex}`
  );
  code = code.replaceAll(/(?<== )t(?=\d)/g, `_t${state.preambleIndex}`).replaceAll("_ctx: any", "").replaceAll("$event: any", "$event");
  state.preambleIndex++;
  for (const [, key, value] of preamble.matchAll(
    /const (_t\d+) = (_template\(.*\))/g
  )) {
    const result = state.preambleMap.get(value);
    if (result) {
      code = code.replaceAll(key, result);
    } else {
      state.preambleMap.set(value, key);
    }
  }
  for (const [, events] of preamble.matchAll(/_delegateEvents\((.*)\)/g)) {
    events.split(", ").forEach((event) => state.delegateEventSet.add(event));
  }
  const ast = parse(code, {
    sourceFilename: state.filename,
    startLine: node.loc.start.line - 1,
    plugins: ["jsx"]
  });
  if (map) {
    const consumer = new SourceMapConsumer(map);
    const line = node.loc.start.line - 1;
    traverse(ast, {
      Identifier({ node: id }) {
        const originalLoc = consumer.originalPositionFor({
          ...id.loc.start,
          line: id.loc.start.line - line + 1
        });
        const column = originalLoc.line === 1 ? node.loc.start.column : 0;
        if (originalLoc.column) {
          id.loc.start.line = line + originalLoc.line + (path.hub ? 0 : 1);
          id.loc.start.column = column + originalLoc.column;
          id.loc.end.line = line + originalLoc.line;
          id.loc.end.column = column + originalLoc.column + id.name.length;
        }
      }
    });
  }
  path.replaceWith(ast.program.body[0]);
};
function isJSXElement(node) {
  return !!node && (node.type === "JSXElement" || node.type === "JSXFragment");
}

// src/index.ts
var src_default = () => {
  return {
    name: "Vue JSX Vapor",
    inherits: SyntaxJSX,
    visitor: {
      JSXElement: transformJSX,
      JSXFragment: transformJSX,
      Program: {
        enter: (path, state) => {
          state.importSet = /* @__PURE__ */ new Set();
          state.delegateEventSet = /* @__PURE__ */ new Set();
          state.preambleMap = /* @__PURE__ */ new Map();
          state.preambleIndex = 0;
          state.rootCodes = [];
          const collectRoot = (path2) => {
            if (path2.parent?.type !== "JSXExpressionContainer" && !isJSXElement(path2.parent) || path2.parentPath.parent?.type === "JSXAttribute") {
              state.rootCodes.push(path2.getSource());
            }
          };
          path.traverse({
            JSXElement: collectRoot,
            JSXFragment: collectRoot
          });
        },
        exit: (path, { delegateEventSet, importSet, preambleMap }) => {
          const statements = [];
          if (delegateEventSet.size) {
            statements.unshift(
              `_delegateEvents(${Array.from(delegateEventSet).join(", ")});`
            );
          }
          if (preambleMap.size) {
            let preambleResult = "";
            for (const [value, key] of preambleMap) {
              preambleResult += `const ${key} = ${value}
`;
            }
            statements.unshift(preambleResult);
          }
          const helpers = ["setText", "createTextNode"].filter((helper) => {
            const result = importSet.has(helper);
            result && importSet.delete(helper);
            return result;
          });
          if (helpers.length) {
            statements.unshift(
              `import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'unplugin-vue-jsx-vapor/helper.js';
`
            );
          }
          if (importSet.size) {
            const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
            statements.unshift(`import { ${importResult} } from 'vue/vapor';
`);
          }
          path.node.body.unshift(
            ...parse2(statements.join("\n"), { sourceType: "module" }).program.body
          );
        }
      }
    }
  };
};
export {
  src_default as default
};
